package rlemaigr.classdiagrameditor;

import java.util.EventObject;

import org.eclipse.ui.views.contentoutline.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.gef.commands.*;
import org.eclipse.gef.*;

import rlemaigr.classdiagrameditor.editparts.*;
import rlemaigr.classdiagrameditor.figures.*;
import rlemaigr.classdiagrameditor.model.*;
import org.eclipse.gef.ui.actions.*;

public class DiagramOutlinePage extends ContentOutlinePage implements CommandStackListener
{
	private EditPart input;
	private ActionRegistry ar;
	private CommandStack commandStack;
	
	public DiagramOutlinePage(CommandStack commandStack, EditPart input, ActionRegistry ar)
	{
		this.commandStack = commandStack;
		commandStack.addCommandStackListener(this);
		this.input = input;
		this.ar = ar;
	}
	public void createControl(Composite parent)
	{
		super.createControl(parent);
		
		TreeViewer treeViewer = getTreeViewer();
		treeViewer.setContentProvider(new MyTreeContentProvider2());
		treeViewer.setLabelProvider(new MyLabelProvider2());
		treeViewer.setInput(input);
		treeViewer.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
		
		ContextMenuProviderImpl menuManager = new ContextMenuProviderImpl(ar,null);
		getControl().setMenu(menuManager.createContextMenu(getControl()));
	}
	public void commandStackChanged(EventObject event)
	{
		//mthode de barbarre. yahaaaaaaaaaaaaaa !!!
		getTreeViewer().refresh();
	}
	public void dispose()
	{
		super.dispose();
		commandStack.removeCommandStackListener(this);
	}
}

class MyTreeContentProvider2 implements ITreeContentProvider
{
	public Object[] getChildren(Object parentElement)
	{
		if (parentElement instanceof EditPart)
			return ((EditPart) parentElement).getChildren().toArray();
		else
			return new Object[0];
	}
	
	public Object getParent(Object element)
	{
		if (element instanceof EditPart)
			return ((EditPart) element).getParent();
		else
			return new Object[0];
	}
	public boolean hasChildren(Object element)
	{
		return !(element instanceof MethodEditPart || element instanceof AttributeEditPart);
	}
	public Object[] getElements(Object inputElement)
	{
		return getChildren(inputElement);
	}
	public void dispose()
	{
	}
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
	{
	}
}

class MyLabelProvider2 implements ILabelProvider
{

	public Image getImage(Object element)
	{
		if (element instanceof EditPart)
		{
			Object model = ((EditPart) element).getModel();
			if (model instanceof MethodModel)
				return MethodFigure.getVisibilityIcon(((MethodModel) model).getVisibility());
			else if (model instanceof AttributeModel)
				return AttributeFigure.getVisibilityIcon(((AttributeModel) model).getVisibility());
			else if (model instanceof ClassModel)
				return ClassFigureBorder.IMAGE;
		}
		return null;
	}

	public String getText(Object element)
	{
		if (element instanceof EditPart)
		{
			Object model = ((EditPart) element).getModel();
			if (model instanceof MethodModel)
				return model.toString();
			else if (model instanceof AttributeModel)
				return model.toString();
			else if (model instanceof ClassModel)
				return ((ClassModel) model).getName();
			else if (model instanceof MethodListModel)
				return "mthodes:";
			else if (model instanceof AttributeListModel)
				return "attributs:";
		}
		return null;
	}

	public void addListener(ILabelProviderListener listener)
	{
	}
	public void dispose()
	{
	}
	public boolean isLabelProperty(Object element, String property)
	{
		return false;
	}
	public void removeListener(ILabelProviderListener listener)
	{
	}
}