package rlemaigr.classdiagrameditor;

import rlemaigr.classdiagrameditor.actions.*;
import rlemaigr.classdiagrameditor.model.*;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.*;

import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.parts.*;
import org.eclipse.gef.ui.actions.*;
import org.eclipse.gef.palette.*;
import org.eclipse.gef.editparts.*;
import org.eclipse.gef.*;
import org.eclipse.gef.commands.*;
import org.eclipse.gef.requests.SimpleFactory;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.*;

import org.eclipse.draw2d.geometry.*;
import org.eclipse.jface.action.*;
import org.eclipse.ui.*;
import org.eclipse.ui.views.properties.*;
import org.eclipse.ui.views.contentoutline.*;

import java.util.*;

public class MyEditor extends GraphicalEditorWithPalette
{
	private PropertySheetPage propPage;
	private DiagramModel content;

	public MyEditor()
	{
		setEditDomain(new DefaultEditDomain(this));
	}

	protected PaletteRoot getPaletteRoot()
	{
		PaletteRoot root = new PaletteRoot();
		root.add(new SelectionToolEntry());
		root.add(new MarqueeToolEntry());
		PaletteGroup group = new PaletteGroup("creation tools");
		root.add(group);
		CreationToolEntry createClassEntry =
			new CreationToolEntry(
				"classe",
				"permet de crer une nouvelle classe",
				new SimpleFactory(ClassModel.class),
				SharedImages.DESC_SELECTION_TOOL_16,
				SharedImages.DESC_SELECTION_TOOL_16);
		ConnectionCreationToolEntry connectionToolEntry =
			new ConnectionCreationToolEntry(
				"hritage",
				"permet de crer une nouvelle relation d'hritage",
				new SimpleFactory(InheritanceModel.class),
				SharedImages.DESC_SELECTION_TOOL_16,
				SharedImages.DESC_SELECTION_TOOL_16);
		group.add(0, createClassEntry);
		group.add(1, connectionToolEntry);
		return root;
	}

	protected void configureGraphicalViewer()
	{
		//root editpart:
		FreeformGraphicalRootEditPart root = new FreeformGraphicalRootEditPart();
		getGraphicalViewer().setRootEditPart(root);

		//menu contextuel:
		getGraphicalViewer().setContextMenu(new ContextMenuProviderImpl(getActionRegistry(), getGraphicalViewer()));

		//keyhandler:
		KeyHandler keyHandler = new KeyHandler();
		keyHandler.put(KeyStroke.getPressed(SWT.DEL, 127, 0), getActionRegistry().getAction(GEFActionConstants.DELETE));

		keyHandler.put(KeyStroke.getPressed('a', 0), getActionRegistry().getAction(CreateAttributeAction.ID));
		keyHandler.put(KeyStroke.getPressed('m', 0), getActionRegistry().getAction(CreateMethodAction.ID));
		getGraphicalViewer().setKeyHandler(new GraphicalViewerKeyHandler(getGraphicalViewer()).setParent(keyHandler));
	}
	protected void initializeGraphicalViewer()
	{
		getGraphicalViewer().setEditPartFactory(new EditPartFactoryImpl());
		getGraphicalViewer().setContents(getContents());
	}
	protected DiagramModel getContents()
	{
		if (content == null)
			content = createContents();
		return content;
	}
	protected DiagramModel createContents()
	{
		AttributeModel am1 = new AttributeModel(AttributeModel.PUBLIC, "int", "attribut1");
		AttributeModel am2 = new AttributeModel(AttributeModel.PROTECTED, "String", "attribut2");
		AttributeModel am3 = new AttributeModel(AttributeModel.PROTECTED, "String", "attribut3");
		AttributeModel am4 = new AttributeModel(AttributeModel.PRIVATE, "String", "attribut4");
		AttributeModel am5 = new AttributeModel(AttributeModel.PROTECTED, "String", "attribut5");

		List paramList = new ArrayList();
		paramList.add("int");
		paramList.add("ClassA");
		MethodModel mm1 = new MethodModel(MethodModel.PRIVATE, "String", "methode1", paramList);
		List paramList2 = new ArrayList();
		paramList2.add("String");
		paramList2.add("ClassV");
		MethodModel mm2 = new MethodModel(MethodModel.PRIVATE, "ClassB", "methode2", paramList2);

		AttributeListModel al1 = new AttributeListModel();
		al1.addElement(am1);
		al1.addElement(am2);
		al1.addElement(am3);

		MethodListModel ml1 = new MethodListModel();
		ml1.addElement(mm1);

		ClassModel class1 = new ClassModel();
		class1.setAttributeList(al1);
		class1.setMethodList(ml1);
		class1.setName("Classe1");
		class1.setLocation(new Point(20, 20));
		class1.setSize(new Dimension(200, 170));

		AttributeListModel al2 = new AttributeListModel();
		al2.addElement(am4);
		al2.addElement(am5);

		MethodListModel ml2 = new MethodListModel();
		ml2.addElement(mm2);

		ClassModel class2 = new ClassModel();
		class2.setAttributeList(al2);
		class2.setMethodList(ml2);
		class2.setName("Classe2");
		class2.setLocation(new Point(250, 250));
		class2.setSize(new Dimension(200, 170));

		InheritanceModel in = new InheritanceModel();
		in.setSource(class2);
		in.setTarget(class1);

		DiagramModel diagram = new DiagramModel();
		diagram.addElement(class1);
		diagram.addElement(class2);

		return diagram;
	}

	public void createActions()
	{
		super.createActions();

		ActionRegistry registry = getActionRegistry();
		IAction action;

		action = new CreateAttributeAction((IWorkbenchPart) this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

		action = new CreateMethodAction((IWorkbenchPart) this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

	}
	public void createPartControl(Composite parent)
	{
		super.createPartControl(parent);

		getEditorSite().getActionBars().setGlobalActionHandler(
			IWorkbenchActionConstants.UNDO,
			getActionRegistry().getAction(GEFActionConstants.UNDO));

		getEditorSite().getActionBars().setGlobalActionHandler(
			IWorkbenchActionConstants.REDO,
			getActionRegistry().getAction(GEFActionConstants.REDO));
			
		getEditorSite().getActionBars().setGlobalActionHandler(
			IWorkbenchActionConstants.DELETE,
			getActionRegistry().getAction(GEFActionConstants.DELETE));
	}
	public Object getAdapter(Class aClass)
	{
		if (aClass == CommandStack.class)
			return getCommandStack();
		else if (aClass == IPropertySheetPage.class)
			return getPropertySheetPage();
		else if (aClass == IContentOutlinePage.class)
			return createContentOutlinePage();
		return super.getAdapter(aClass);
	}
	public DiagramOutlinePage createContentOutlinePage()
	{
		if (getGraphicalViewer() != null && getGraphicalViewer().getContents() != null && getActionRegistry() != null)
		{
			DiagramOutlinePage page =
				new DiagramOutlinePage(getCommandStack(), getGraphicalViewer().getContents(), getActionRegistry());
			return page;
		}
		return null;
	}
	public PropertySheetPage getPropertySheetPage()
	{
		if (propPage == null)
		{
			propPage = new PropertySheetPage();
			propPage.setRootEntry(GEFPlugin.createUndoablePropertySheetEntry(getCommandStack()));
		}
		return propPage;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	public void doSave(IProgressMonitor monitor)
	{
	}
	public void doSaveAs()
	{
	}
	public void gotoMarker(IMarker marker)
	{
	}
	public boolean isDirty()
	{
		return false;
	}
	public boolean isSaveAsAllowed()
	{
		return false;
	}

}
