package rlemaigr.classdiagrameditor.model;

import org.eclipse.draw2d.geometry.*;
import rlemaigr.classdiagrameditor.misc.*;
import org.eclipse.ui.views.properties.*;

public class DiagramElementModel extends ObjectModel
{
	private Point location;
	private Dimension size;

	private static IPropertyDescriptor[] descriptors;

	public static final String PROPERTY_CONSTRAINT_CHANGED = "constraint";

	public static final String ID_PROPERTY_LOCATION_X = "location x";
	public static final String ID_PROPERTY_LOCATION_Y = "location y";
	public static final String ID_PROPERTY_HEIGHT = "height";
	public static final String ID_PROPERTY_WIDTH = "width";

	public void setLocation(Point location)
	{
		Point save = getLocation();
		this.location = location;
		firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, save, location);
	}
	public Point getLocation()
	{
		return location;
	}
	public void setSize(Dimension size)
	{
		Dimension save = getSize();
		this.size = size;
		firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, save, size);
	}
	public Dimension getSize()
	{
		return size;
	}
	public void setConstraint(Rectangle rect)
	{
		Rectangle save = new Rectangle(getLocation(), getSize());
		this.size = rect.getSize();
		this.location = rect.getLocation();
		firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, save, rect);
	}
	public Rectangle getConstraint()
	{
		return new Rectangle(getLocation(), getSize());
	}
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
			IPropertyDescriptor[] superDescriptors = super.getPropertyDescriptors();
			descriptors = new IPropertyDescriptor[superDescriptors.length + 4];
			for (int i = 0; i < superDescriptors.length; i++)
				descriptors[i] = superDescriptors[i];
			descriptors[superDescriptors.length] = new IntegerPropertyDescriptor(ID_PROPERTY_LOCATION_X, "x");
			descriptors[superDescriptors.length + 1] = new IntegerPropertyDescriptor(ID_PROPERTY_LOCATION_Y, "y");
			descriptors[superDescriptors.length + 2] = new IntegerPropertyDescriptor(ID_PROPERTY_HEIGHT, "height");
			descriptors[superDescriptors.length + 3] = new IntegerPropertyDescriptor(ID_PROPERTY_WIDTH, "width");
		}
		return descriptors;
	}
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_LOCATION_X))
				setLocation(new Point(Integer.parseInt(val.toString()), getLocation().y));
			else if (((String) propName).equals(ID_PROPERTY_LOCATION_Y))
				setLocation(new Point(getLocation().x, Integer.parseInt(val.toString())));
			else if (((String) propName).equals(ID_PROPERTY_WIDTH))
				setSize(new Dimension(Integer.parseInt(val.toString()), getSize().height));
			else if (((String) propName).equals(ID_PROPERTY_HEIGHT))
				setSize(new Dimension(getSize().width, Integer.parseInt(val.toString())));
		}
		else
			super.setPropertyValue(propName, val);
	}
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_LOCATION_X))
				return new Integer(getLocation().x).toString();
			else if (((String) propName).equals(ID_PROPERTY_LOCATION_Y))
				return new Integer(getLocation().y).toString();
			else if (((String) propName).equals(ID_PROPERTY_WIDTH))
				return new Integer(getSize().width).toString();
			else if (((String) propName).equals(ID_PROPERTY_HEIGHT))
				return new Integer(getSize().height).toString();
		}
		return super.getPropertyValue(propName);
	}
}