package rlemaigr.classdiagrameditor.model;

import java.util.*;
import org.eclipse.ui.views.properties.*;

public class MethodModel extends ObjectModel
{
	private String identifier;
	private String returnType;
	private int visibility;
	private List parametersTypes;
	private IPropertyDescriptor[] descriptors;

	public static final String PROPERTY_SIGNATURE_CHANGED = "signature";
	public static final int PUBLIC = 0;
	public static final int PRIVATE = 1;
	public static final int PROTECTED = 2;

	public static final String ID_PROPERTY_IDENTIFIER = "identifier";
	public static final String ID_PROPERTY_RETURN_TYPE = "return type";
	public static final String ID_PROPERTY_VISIBILITY = "visibilit";

	public MethodModel()
	{
		setSignature(PUBLIC, "no_type", "no_identifier", new ArrayList());
	}
	public MethodModel(int visibility, String returnType, String identifier, List parametersTypes)
	{
		setSignature(visibility, returnType, identifier, parametersTypes);
	}
	public void setSignature(int visibility, String returnType, String identifier, List parametersTypes)
	{
		this.visibility = visibility;
		this.returnType = returnType;
		this.identifier = identifier;
		this.parametersTypes = parametersTypes;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	public int getVisibility()
	{
		return visibility;
	}
	public String getReturnType()
	{
		return returnType;
	}
	public String getIdentifier()
	{
		return identifier;
	}
	public List getParametersTypes()
	{
		return parametersTypes;
	}
	public String toString()
	{
		String txt = returnType + " " + identifier + "(";
		Iterator it = parametersTypes.iterator();
		if (it.hasNext())
			txt += it.next().toString();
		while (it.hasNext())
		{
			txt += " ," + it.next().toString();
		}
		txt += ")";
		return txt;
	}
	public void setReturnType(String returnType)
	{
		this.returnType = returnType;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	public void setIdentifier(String identifier)
	{
		this.identifier = identifier;
		firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
	}
	public void setVisibility(int visibility)
	{
		if (visibility >= 0 && visibility < 3)
		{
			this.visibility = visibility;
			firePropertyChange(PROPERTY_SIGNATURE_CHANGED, null, null);
		}
	}
	public IPropertyDescriptor[] getPropertyDescriptors()
	{
		if (descriptors == null)
		{
			IPropertyDescriptor[] superDescriptors = super.getPropertyDescriptors();
			descriptors = new IPropertyDescriptor[superDescriptors.length + 3];
			for (int i = 0; i < superDescriptors.length; i++)
				descriptors[i] = superDescriptors[i];
			descriptors[superDescriptors.length] =
				new TextPropertyDescriptor(ID_PROPERTY_RETURN_TYPE, "type du retour");
			descriptors[superDescriptors.length + 1] =
				new TextPropertyDescriptor(ID_PROPERTY_IDENTIFIER, "identificateur");
			descriptors[superDescriptors.length + 2] =
				new ComboBoxPropertyDescriptor(
					ID_PROPERTY_VISIBILITY,
					"visibilit",
					new String[] { "public", "protected", "private" });
		}
		return descriptors;
	}
	public void setPropertyValue(Object propName, Object val)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				setReturnType(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				setIdentifier(val.toString());
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (((Integer) val).intValue() == 0)
					setVisibility(PUBLIC);
				else if (((Integer) val).intValue() == 2)
					setVisibility(PRIVATE);
				else if (((Integer) val).intValue() == 1)
					setVisibility(PROTECTED);
			}
		}
		else
			super.setPropertyValue(propName, val);
	}
	public Object getPropertyValue(Object propName)
	{
		if (propName instanceof String)
		{
			if (((String) propName).equals(ID_PROPERTY_RETURN_TYPE))
				return returnType;
			else if (((String) propName).equals(ID_PROPERTY_IDENTIFIER))
				return identifier;
			else if (((String) propName).equals(ID_PROPERTY_VISIBILITY))
			{
				if (visibility == PUBLIC)
					return new Integer(0);
				else if (visibility == PRIVATE)
					return new Integer(2);
				else if (visibility == PROTECTED)
					return new Integer(1);
				else
					return super.getPropertyValue(propName);
			}
			else
				return super.getPropertyValue(propName);
		}
		else
			return super.getPropertyValue(propName);
	}
}